﻿using IRaCIS.Application.Interfaces;
using IRaCIS.Core.Infra.EFCore;
using IRaCIS.Core.Domain.Share;
using IRaCIS.Core.Application.Filter;
using Microsoft.AspNetCore.Mvc;
using IRaCIS.Core.Application.Service.WorkLoad.DTO;
using Microsoft.AspNetCore.Authorization;
using IRaCIS.Core.Application.Auth;
using IRaCIS.Core.Application.Service.Reading.Dto;
using IRaCIS.Core.Domain.Share.Reading;
using MassTransit;

namespace IRaCIS.Application.Services
{
    /// <summary>
    /// 临床数据配置
    /// </summary>
    [ApiExplorerSettings(GroupName = "Reading")]
    public class ClinicalDataSetService : BaseService
    {

        public IRepository<SubjectVisit> _subjectVisitRepository;
        private readonly IRepository<ClinicalData> _clinicalDataRepository;
        private readonly IRepository<ClinicalDataSet> _clinicalDataSetRepository;
        private readonly IRepository<PreviousPDF> _previousPDFRepository;

        public ClinicalDataSetService(IRepository<SubjectVisit> subjectVisitRepository,
            IRepository<ClinicalData> clinicalDataRepository,
            IRepository<ClinicalDataSet> ClinicalDataSetRepository,
            IRepository<PreviousPDF> previousPDFRepository
          )
        {
            _subjectVisitRepository = subjectVisitRepository;
            _clinicalDataRepository = clinicalDataRepository;
            _clinicalDataSetRepository = ClinicalDataSetRepository;
            this._previousPDFRepository = previousPDFRepository;
        }


        /// <summary>
        /// 新增或者修改
        /// </summary>
        /// <param name="indto"></param>
        /// <returns></returns>
        [HttpPost]
        public async Task<IResponseOutput> AddOrUpdateClinicalDataSet(ClinicalDataSetAddOrEdit indto)
        {
            var entity = await _clinicalDataSetRepository.InsertOrUpdateAsync(indto,true);
            return ResponseOutput.Ok(entity.Id.ToString());
        }

        /// <summary>
        /// 分页获取
        /// </summary>
        /// <param name="query"></param>
        /// <returns></returns>
        [HttpPost]
        public async Task<PageOutput<ClinicalDataSetView>> GetClinicalDataSetList(ClinicalDataQuery query)
        {
            var readquery = _clinicalDataSetRepository.AsQueryable()
               .Where( x => x.TrialId == query.TrialId)
               .WhereIf(query.ClinicalDataSetName!=string.Empty,x=>x.ClinicalDataSetName.Contains(query.ClinicalDataSetName))
               .ProjectTo<ClinicalDataSetView>(_mapper.ConfigurationProvider);
            var pageList = await readquery.ToPagedListAsync(query.PageIndex, query.PageSize, query.SortField == null ? "CreateTime" : query.SortField,
           query.Asc);

            return pageList;
        }

        /// <summary>
        /// 获取系统临床数据
        /// </summary>
        /// <returns></returns>
        [HttpGet]
        public async Task<List<ClinicalDataSetView>> GetSystemClinicalDataSetList()
        {
            return await _clinicalDataSetRepository.AsQueryable()
             .Where(x => x.TrialId == null)
             .ProjectTo<ClinicalDataSetView>(_mapper.ConfigurationProvider).ToListAsync();
        }

        public async Task<List<ClinicalDataSetView>> GetTrialClinicalDataSetList(AddTrialClinicalDataSetInDto inDto)
        {
            await this.AddTrialClinicalDataSet(inDto);

            return await _clinicalDataSetRepository.AsQueryable()
               .Where(x => x.TrialId == inDto.TrialId)
               .ProjectTo<ClinicalDataSetView>(_mapper.ConfigurationProvider).ToListAsync();

        }


        /// <summary>
        /// 将系统配置添加到项目配置
        /// </summary>
        /// <param name="inDto"></param>
        /// <returns></returns>

        private async Task<IResponseOutput> AddTrialClinicalDataSet(AddTrialClinicalDataSetInDto inDto)
        {
            var systemIds = await _clinicalDataSetRepository.Where(x => x.TrialId == null).Select(x => x.Id).ToListAsync();
            var trialSystemIds =await _clinicalDataSetRepository.Where(x => x.TrialId == inDto.TrialId && x.SystemClinicalDataSetId != null).Select(x => x.SystemClinicalDataSetId.Value).ToListAsync();
            var needAddids  = systemIds.Except(trialSystemIds).ToList();
            var systemDataList =await _clinicalDataSetRepository.Where(x => needAddids.Contains(x.Id)).ToListAsync();
            List<ClinicalDataSet> dataSets = systemDataList.Select(x => new ClinicalDataSet()
            {
                Id= NewId.NextGuid(),
                SystemClinicalDataSetId=x.Id,
                ClinicalDataSetName=x.ClinicalDataSetName,
                ClinicalDataLevel=x.ClinicalDataLevel,
                ClinicalUploadType=x.ClinicalUploadType,
                TrialId=inDto.TrialId,
            }).ToList();
            await _clinicalDataSetRepository.AddRangeAsync(dataSets);
            var result=  await _clinicalDataSetRepository.SaveChangesAsync();
            return ResponseOutput.Ok(result);
        }




        /// <summary>
        /// 删除
        /// </summary>
        /// <param name="clinicalDataId"></param>
        /// <returns></returns>
        [HttpDelete("{id:guid}")]
        public async Task<IResponseOutput> DeleteClinicalSetData(Guid id)
        {
           await _clinicalDataSetRepository.DeleteFromQueryAsync(x=>x.Id== id,true);
            return ResponseOutput.Result(true);
        }
    }
}
